# This file generate the figures of the manuscript.

library(ggplot2)
library(RColorBrewer)
library(cowplot)
library(ape)
library(ggplotify)
library(reshape2)
library(plyr)
library(ggpubr)
library(tibble)
library(ggforce)
library(arules)
library(dplyr)
library(lemon)

#### Please set your working directory ####
# setwd(dirname(rstudioapi::getSourceEditorContext()$path))


source("add_pv.R")

# Load the names dataframe
names_df <- read.table("data/names.tsv")
names(names_df) <- c("name","NAME")

###################
#### FUNCTIONS ####
###################
plot_violins_with_dots <- function(data, feature){
    # Create separate data frames for the points and violins
    points_data <- data
    violins_data <- data

    # Define colors
    colors <- c("darkorchid","#ac7ba8","#b49b7f","darkorange","darkgrey","grey","lightgrey","#c7b299","#c2a076","#fda232")
    names(colors) <- c("IORF","NCOS_orphan","NCOS_non_orphan","nonTRG","NCO3S","NCO2S","NCOS","NCO","NC","TRG")
    colors_to_use <- colors[levels(points_data$variable)]

    print(table(points_data$variable))
    # Create the plot
    p <- ggplot() +
      geom_violin(data = violins_data, aes(x = variable, y = {{feature}}, colour=variable), alpha = 0.2) +
      # scale_colour_manual(values = c("#300e41","#5c4325","#552f00","grey20")) +  # Darker border color for violins
      geom_sina(data = points_data, aes(x = variable, y = {{feature}}, fill=variable, colour = variable), shape = 21, size = 0.125) +
      scale_fill_manual(values = colors_to_use) +  # Fill color for points
      scale_colour_manual(values = colors_to_use) +  # Same border color as fill for points
      stat_summary(data = points_data, aes(x = variable, y = {{feature}}), fun = "median",
                    geom = "crossbar",
                    width = 0.5,  # Reduce the width
                    lwd = 0.125)  # Reduce the line's thickness
    return(p)
}
create_transformation <- function(threshold) {
  #' Create a transformation for compressing values above a threshold
  #'
  #' This function creates a transformation that can be used with ggplot2's scale functions.
  #' Values above the specified threshold are compressed using a square root transformation,
  #' while values below the threshold are left unchanged.
  #'
  #' @param threshold A numeric value specifying the threshold above which values should be compressed.
  #'
  #' @return A trans object that can be used as the trans argument to ggplot2's scale functions.
  #'
  #' @examples
  #' trans <- create_transformation(500)
  #' breaks <- c(seq(0, 500, by = 50), seq(1000, 7500, by = 1000))
  #' ggplot(data, aes(x, y)) +
  #'   geom_point() +
  #'   scale_y_continuous(trans = trans, breaks = breaks)

  # Define the custom transformation
  compress_above_threshold <- function(x) {
    sapply(x, function(y) ifelse(y > threshold, threshold + sqrt(y - threshold), y))
  }
  compress_above_threshold_inv <- function(x) {
    ifelse(x > threshold, threshold + (x - threshold)^2, x)
  }

  # Register the transformation
  trans <- scales::trans_new(
    paste0("compress_above_", threshold), 
    transform = compress_above_threshold, 
    inverse = compress_above_threshold_inv
  )

  return(trans)
}
get_exclusive_plot <- function( NAME,
                                feature,
                                groups,
                                comps,
                                yp = NULL,
                                comp_min_group_size = 30,
                                signif_size = 3,
                                num_outgroups = FALSE
                                ){
  #' get_exclusive_plot function
  #'
  #' Returns a plot for a given genome and a given feature.
  #'
  #' @param genome The genome to plot.
  #' @param feature The feature to plot.
  #' @return A plot object.
  
  # Check that the genome and the feature are valid.
  # if (! NAME %in% names_df$NAME) {
  #   stop(paste(NAME, " is not a valid NAME (possible values : ", paste(as.character(names_df$NAME)),"."))
  # }
  if (! feature %in% c("size", "nb_exons", "GAN", "p_neg","gene_dist","age")) {
    stop(paste(feature, " is not a valid feature (possible values : 'size', 'nb_exons', 'GAN', 'p_neg', 'gene_dist','age')."))
  }
  
  # name <- as.character(names_df[names_df$NAME == NAME, "name"])
  name <- "Oryza_sativa"
  print(name)
  
  # Load the dataframe with the different groups as columns and sequences as rows.
  if (num_outgroups){
    main_df <- read.table(paste("data/",NAME,"_main_dataframe_nco3s.tsv", sep=""), header = TRUE)

    # Build a dataframe with mutually exclusive groups of TRG.
    exclusive_df <- main_df
    exclusive_df$NCO3S <- main_df$NCO3S
    exclusive_df$NCO2S <- main_df$NCO2S & ! main_df$NCO3S
    exclusive_df$NCOS <- main_df$NCOS & ! main_df$NCO2S & ! main_df$NCO3S
    exclusive_df$NCO <- main_df$NCO & ! main_df$NCOS & ! main_df$NCO2S & ! main_df$NCO3S
    exclusive_df$NC <- main_df$NC & ! main_df$NCO & ! main_df$NCOS & ! main_df$NCO2S & ! main_df$NCO3S
    exclusive_df$TRG <- main_df$TRG & ! main_df$NC & ! main_df$NCO & ! main_df$NCOS & ! main_df$NCO2S & ! main_df$NCO3S

    # Check that the sum of the different groups is equal to the number of sequences.
    total_main <- sum(main_df$TRG == "TRUE", na.rm = TRUE)
    total_exclusive <- sum(exclusive_df$NCO3S == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCO2S == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCO == "TRUE", na.rm = TRUE) + sum(exclusive_df$NC == "TRUE", na.rm = TRUE) + sum(exclusive_df$TRG == "TRUE", na.rm = TRUE)
    if(total_main != total_exclusive){
      print("total main : ")
      print(total_main)
      print("total exclusive : ")
      print(total_exclusive)
      stop("The sum of the different groups is not equal to the number of sequences.")
    }

    # Check that the sum of the different groups is equal to the number of sequences.
    total_main <- sum(main_df$TRG == "TRUE", na.rm = TRUE)
    total_exclusive <- sum(exclusive_df$NCO3S == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCO2S == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCO == "TRUE", na.rm = TRUE) + sum(exclusive_df$NC == "TRUE", na.rm = TRUE) + sum(exclusive_df$TRG == "TRUE", na.rm = TRUE)
    if(total_main != total_exclusive){
      print("total main : ")
      print(total_main)
      print("total exclusive : ")
      print(total_exclusive)
      stop("The sum of the different groups is not equal to the number of sequences.")
    }

    # Add the orphan information
    exclusive_df$NCOS_orphan <- (exclusive_df$NCOS | exclusive_df$NCO2S | exclusive_df$NCO3S) & exclusive_df$age == 0
    exclusive_df$NCOS_non_orphan <- (exclusive_df$NCOS | exclusive_df$NCO2S | exclusive_df$NCO3S) & exclusive_df$age > 0

    # Check that the sum of the NCOS orphan and non-orphan is equal to the number of NCOS.
    if(sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE)+sum(exclusive_df$NCO2S == "TRUE", na.rm = TRUE)+sum(exclusive_df$NCO3S == "TRUE", na.rm = TRUE) != sum(exclusive_df$NCOS_orphan == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCOS_non_orphan == "TRUE", na.rm = TRUE)){
      print("total NCOS : ")
      print(sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE)+sum(exclusive_df$NCO2S == "TRUE", na.rm = TRUE)+sum(exclusive_df$NCO3S == "TRUE", na.rm = TRUE))
      print("total NCOS orphan and non-orphan : ")
      print(sum(exclusive_df$NCOS_orphan == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCOS_non_orphan == "TRUE", na.rm = TRUE))
      stop("The sum of the NCOS orphan and non orphan is not equal to the number of NCOS.")
    }




  } else {
    # Load the dataframe with the different groups as columns and sequences as rows. 
    main_df <- read.table(paste("data/",NAME,"_main_dataframe.tsv", sep=""), header = TRUE)
    
    # Build a dataframe with mutually exclusive groups of TRG.
    exclusive_df <- main_df
    exclusive_df$NCOS <- main_df$NCOS
    exclusive_df$NCO <- main_df$NCO & ! main_df$NCOS
    exclusive_df$NC <- main_df$NC & ! main_df$NCO & ! main_df$NCOS
    exclusive_df$TRG <- main_df$TRG & ! main_df$NC & ! main_df$NCO & ! main_df$NCOS

    # Check that the sum of the different groups is equal to the number of sequences.
    total_main <- sum(main_df$TRG == "TRUE", na.rm = TRUE)
    total_exclusive <- sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCO == "TRUE", na.rm = TRUE) + sum(exclusive_df$NC == "TRUE", na.rm = TRUE) + sum(exclusive_df$TRG == "TRUE", na.rm = TRUE)
    if(total_main != total_exclusive){
      print("total main : ")
      print(total_main)
      print("total exclusive : ")
      print(total_exclusive)
      stop("The sum of the different groups is not equal to the number of sequences.")
    }



    # Add the orphan information
    exclusive_df$NCOS_orphan <- exclusive_df$NCOS & exclusive_df$age == 0
    exclusive_df$NCOS_non_orphan <- exclusive_df$NCOS & exclusive_df$age > 0

    # Check that the sum of the NCOS orphan and non-orphan is equal to the number of NCOS.
    if(sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE) != sum(exclusive_df$NCOS_orphan == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCOS_non_orphan == "TRUE", na.rm = TRUE)){
      print("total NCOS : ")
      print(sum(exclusive_df$NCOS == "TRUE", na.rm = TRUE))
      print("total NCOS orphan and non-orphan : ")
      print(sum(exclusive_df$NCOS_orphan == "TRUE", na.rm = TRUE) + sum(exclusive_df$NCOS_non_orphan == "TRUE", na.rm = TRUE))
      stop("The sum of the NCOS orphan and non orphan is not equal to the number of NCOS.")
    }
  }

  exclusive_df$nonTRG <- main_df$TRG == FALSE & ! is.na(main_df$TRG) & exclusive_df$seq_type == "CDS"
  exclusive_df$IORF <- exclusive_df$seq_type == "IORF"

  # Build the plotable data frame
  melted_df <- reshape2::melt(data = exclusive_df, measure.vars = groups)
  melted_df <- melted_df[melted_df$value == "TRUE" & ! is.na(melted_df$value), ]
  melted_df$variable <- factor(melted_df$variable,
                               levels = groups,
                               ordered = TRUE)
  print(table(melted_df$variable))


  # Remove (near-)empty groups
  melted_df <- melted_df[!is.na(melted_df[ ,feature]), ]
  contingents <- table(melted_df$variable)
  print("Groups non NA")
  print(contingents)
  empties <- names(contingents[contingents <= 1])
  if (length(empties) > 0) {
    print("Empty groups : ")
    print(empties)
    melted_df <- melted_df[!melted_df$variable %in% empties, ]
  }
  # Remove comparisons that include empty groups
  comps <- comps[!sapply(comps, function(x) any(x %in% names(contingents[contingents <= comp_min_group_size])))]
  if (length(yp) > length(comps)) {
    yp <- yp[1:length(comps)]
  }


  # Generate the plot.
  if(feature == "size"){
    p <- plot_violins_with_dots(data = melted_df, feature = size)+
      coord_cartesian(ylim = c(0,7500))
      if(length(comps) > 0){
        p <- p + stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                        comps = comps,
                                                        col.x = "variable",
                                                        col.y = "size"), tip.length = 0, y.position = yp, size = signif_size)
      }
      
                                                        
    # Change p's y axis into a non-linear scale where values above 500 are compressed.
    breaks <- c(seq(0, 500, by = 50), 2000, 10000)
    p <- p + scale_y_continuous(trans = create_transformation(500), breaks = breaks)
  }
  if(feature == "GAN"){
    p <- plot_violins_with_dots(data = melted_df, feature = GAN)+
    coord_cartesian(ylim = c(0,0.3)) +
    if(length(comps) > 0){
    stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                      comps = comps,
                                                      col.x = "variable",
                                                      col.y = "GAN"), tip.length = 0, y.position = yp, size = signif_size)
    }
  }
  if(feature == "gene_dist"){
    p <- plot_violins_with_dots(data = melted_df, feature = gene_dist)+
      coord_cartesian(ylim = c(0,10000))+
      ylab("Dist. to closest gene dist (bp)")
      if(length(comps) > 0){
      p <- p+
      stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                        comps = comps,
                                                        col.x = "variable",
                                                        col.y = "gene_dist"), tip.length = 0, y.position = yp, size = signif_size)
      }

    # Change p's y axis into a non-linear scale where values above 500 are compressed.
    breaks <- c(seq(0, 1250, by = 100), 2000, 20000)
    p <- p + scale_y_continuous(trans = create_transformation(1250), breaks = breaks)
  }
  if(feature == "p_neg"){

    p <- plot_violins_with_dots(data = melted_df, feature = p_neg)+
      coord_cartesian(ylim = c(0,1))+
      ylab("negative selection fraction")
      if(length(comps) > 0){
        p <- p+
          stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                            comps = comps,
                                                            col.x = "variable",
                                                            col.y = "p_neg"), tip.length = 0, y.position = yp, size = signif_size)
      }
      
  }  
  if(feature == "age"){
    p <- plot_violins_with_dots(data = melted_df, feature = age)+
    coord_cartesian(ylim = c(0,1))+
    ylab("Phylogenetic distance")
    if(length(comps) > 0){
      p <- p+
    stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                      comps = comps,
                                                      col.x = "variable",
                                                      col.y = "age"), tip.length = 0, y.position = yp, size = signif_size)
    }
  }
  
  p <- p + ggtitle(NAME) + theme( panel.background = element_blank())
  
  return(p)

}


###############
#### Fig 3 ####
###############


# This section generates the heatmap of the conservation of denovo CDS and IGR in the different species (Fig3).

plots <- list()
correlation_df <- data.frame(NAME = character(), rho = numeric(), p_value = numeric())

# for (NAME in c("SCER","ATHA","OSAT","CELE","DMEL","MMUS")){ # Waiting for HSAP
for (NAME in names_df$NAME){

  # denovo list
  main_df <- read.table(paste("data/",NAME,"_main_dataframe.tsv", sep=""), header = TRUE)
  # denovos <- as.vector(main_df[main_df$NCO == "TRUE" & ! is.na(main_df$NCO), "seq_name"])
  denovos <- as.vector(main_df[main_df$NCOS == "TRUE" & ! is.na(main_df$NCOS), "seq_name"])

  # Match matrix
  match_matrix <- read.table(paste("data/TRG_match_matrix/",NAME,"_TRG_match_matrix.tsv", sep=""), header = TRUE)
  # Remove "_elongated" from the end of match_matrix$CDS
  match_matrix$CDS <- gsub("_elongated", "", match_matrix$CDS)
  # Only keep the denovo genes CDS
  match_matrix <- match_matrix[match_matrix$CDS %in% denovos, ]

  # Replace genome names by short names in the colnames of match matrix
  names_mapping_df <- read.table(paste("data/names/",NAME,"_names.tsv", sep=""), sep = "\t")
  colnames(names_mapping_df) <- c("genome","short")
  # Find the indices of the long names in names_mapping_df$genome
  indices <- match(colnames(match_matrix)[-1], names_mapping_df$genome)
  # Replace the long names with the corresponding short names
  colnames(match_matrix)[-1] <- names_mapping_df$short[indices]
  # Remove the "CDS" column
  match_matrix <- match_matrix[, -1]
  
  # Conservation
  # Calculate the percentage of "CDS" for each genome
  CDS_conservation <- colSums(match_matrix == "CDS") / nrow(match_matrix) * 100

  # Conservation dataframe
  conservation_df <- data.frame( genome = colnames(match_matrix),
                                 CDS = as.numeric(CDS_conservation) )

  # Add the IGRs column
  IGR_conservation_df <- read.csv(paste("data/IGR_conservation/",NAME,"_IGR_conservation_table.csv", sep=""))
  indices <- match(IGR_conservation_df$genome, names_mapping_df$genome)
  IGR_conservation_df$genome <- names_mapping_df$short[indices]
  conservation_df <- merge(conservation_df, IGR_conservation_df, by = "genome")
  print(conservation_df)

  write.table(conservation_df, paste("data/Fig3/",NAME,"_conservation.tsv", sep=""), sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE)  
  # Correlation
  correlation <- cor.test(conservation_df$CDS, conservation_df$IGR_rel, method = "spearman", exact = FALSE)
  
  rho <- correlation$estimate
  p_value <- correlation$p.value
  print(NAME)
  print(paste("Spearman's rho: ", rho))
  print(paste("p-value: ", p_value))
  correlation_df <- rbind(correlation_df, c(NAME, rho, p_value))

  # Tree
  tree_path <- paste("data/trees/",NAME,".nwk", sep="") # OF trees with focal at the top.
  tree <- read.tree(file = tree_path)
  tree$edge.length <- NULL # Remove branch length
  # Use the short names
  tree$tip.label <- names_mapping_df$short[match(tree$tip.label, names_mapping_df$genome)]
  leaf_order <- tree$tip.label

  # Manage labels=genomes=leaves
  for (i in 1:length(leaf_order)){
    if (!(leaf_order[i] %in% conservation_df$genome)){
      conservation_df <- rbind(conservation_df, c(leaf_order[i], 0, 0))
    }
  }
  conservation_df$genome <- factor(conservation_df$genome, levels = leaf_order, ordered = TRUE)
  conservation_df <- conservation_df[match(leaf_order, conservation_df$genome), ]
  
  plot1 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = CDS))+
    geom_tile()+
    scale_fill_gradient2(low="#FAFAFA", high="#202020", limits = c(0, 100)) +
    labs(fill = "conservation of de novo emerged genes") +
    guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
    theme_minimal()+
    theme(legend.position = "bottom",
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 6),
          legend.key.width = unit(1, "cm"),  # Adjust the width
          legend.key.height = unit(0.5, "cm"),  # Adjust the height
          legend.box = "vertical", legend.margin = margin(),
          axis.title = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank(),
          panel.grid = element_blank())+theme(plot.margin = margin(r=0.5))

  legend1 <- cowplot::get_legend(plot1)

  plot1 <- plot1 + theme(legend.position = "none")

  plot2 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = IGR_rel))+
    geom_tile()+
    scale_fill_gradient2(low="#FAFAFA", high="#54067a", limits = c(0, 100)) +
    labs(fill = "conservation of noncoding ORFs") +
    guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
    theme_minimal()+
    theme(legend.position = "bottom",
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 6),
          legend.key.width = unit(1, "cm"),  # Adjust the width
          legend.key.height = unit(0.5, "cm"),  # Adjust the height
          legend.box = "vertical", legend.margin = margin(),
          axis.title = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank(),
          panel.grid = element_blank())+theme(plot.margin = margin(l=0.5))
  
  legend2 <- cowplot::get_legend(plot2)

  plot2 <- plot2 + theme(legend.position = "none")
  
  # Convert the tree to a data frame
  tree_df <- as.data.frame(tree$edge)
  tree_df$Label <- tree$tip.label[tree_df$V2]
  
  # Create a base R plot
  base_plot <- function() {
    plot(tree, no.margin = TRUE, cex = 0.95)
  }
  # Convert the base R plot to a ggplot object
  plot_tree <- as.ggplot(base_plot)#+theme(plot.margin = margin(t=1, r=1, b=1, l=1, "cm"))
  
  plot <- plot_grid(plot_tree,plot1,plot2, nrow = 1, scale = c(1,1,1), rel_widths = c(7, 1, 1))
  plot
  plots[[NAME]] <- plot

}

colnames(correlation_df) <- c("NAME","rho","p_value")
write.table(correlation_df, "data/TableS2.tsv", sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE)

HM_plots <- plots
# HM_plots[["LEGEND"]] <- ggdraw()+draw_label("I am a legend.", size = 10, x = 0.5, y = 0.5)
HM_plots[["LEGEND"]] <- plot_grid(legend1, legend2, nrow = 2, scale = c(1,1), rel_widths = c(1, 1))

# PDF
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig3.pdf"), width = 15)
# for (plot in plots){
#   print(plot)
# }
HM_grid <- plot_grid(HM_plots[["SCER"]],HM_plots[["ATHA"]],HM_plots[["OSAT"]],HM_plots[["CELE"]],
                     NULL, NULL, NULL, NULL,
                     HM_plots[["DMEL"]],HM_plots[["MMUS"]],HM_plots[["HSAP"]],HM_plots[["LEGEND"]], nrow = 3, rel_heights = c(5,1,5))
HM_grid
dev.off()


################
#### Fig 4 #####
################

base_breaks_x <- function(x){
  b <- pretty(x)
  d <- data.frame(y=-Inf, yend=-Inf, x=min(b), xend=max(b))
  list(geom_segment(data=d, aes(x=x, y=y, xend=xend, yend=yend), inherit.aes=FALSE),
       scale_x_continuous(breaks=b))
}
base_breaks_y <- function(x){
  b <- pretty(x)
  d <- data.frame(x=-Inf, xend=-Inf, y=min(b), yend=max(b))
  list(geom_segment(data=d, aes(x=x, y=y, xend=xend, yend=yend), inherit.aes=FALSE),
       scale_y_continuous(breaks=b))
}


#### Fig 4A ####

df <- read.csv("data/Fig4A/FigKL.csv")
df$distance <- df$distance/max(df$distance)
head(df)

# Correlation
correlation <- cor.test(df$nb_NCOS,df$distance, method = "spearman", exact = FALSE)
rho <- correlation$estimate
p_value <- correlation$p.value
print(paste("Spearman's rho: ", rho))
print(paste("p-value: ", p_value))

p <- ggplot(data = df, aes(x = distance, y = nb_NCOS))+
  geom_point()+
  geom_line()+
  coord_cartesian(ylim = c(0,2500))+
  xlab("Distance between the focal and its nearest considered neighbor")+
  ylab("Number of predicted de novo emerged genes")+
  theme_bw() +
  theme(
    panel.border = element_blank(),
    # panel.grid.major = element_blank(),
    # panel.grid.minor = element_blank(),
    axis.text = element_text(size = 14, color = "black"),
    axis.title.x = element_text(size = 14, color = "black", vjust = -2),
    axis.title.y = element_text(size = 14, color = "black", vjust = 5, hjust = 0.25),
    plot.margin = margin(1, 1, 1, 1, "cm"))+
  base_breaks_x(df$distance) +
  base_breaks_y(c(0,df$nb_NCOS))

p

pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig4A.pdf"), width = 8)
p
dev.off()

#### Fig 4B ####

df_CDS_1000_exons <- read.table("data/Fig4B/CDS_1000_qcov.tsv", header = FALSE)
df_oprhan_denovo_exons <- read.table("data/Fig4B/orphan_denovo_qcov.tsv", header = FALSE)
df_iORF_1000_exons <- read.table("data/Fig4B/iORF_1000_qcov.tsv", header = FALSE)

median(df_CDS_1000_exons$V2)
median(df_oprhan_denovo_exons$V2)
median(df_iORF_1000_exons$V2)

group1 <- df_CDS_1000_exons$V2
group2 <- df_oprhan_denovo_exons$V2
pvalues = NULL
if (length(group1) > 500 || length(group2) > 500){
  for (i in 1:100){
    if (length(group1) > 500){
      s1 = sample(group1, size = 500)
    } else {
      s1 = group1
    }
    if (length(group2) > 500){
      s2 = sample(group2, size = 500)
    } else {
      s2 = group2
    }
    pval.tmp = wilcox.test(s1, s2, alternative = "greater", exact = FALSE)$p.value
    pvalues = c(pvalues,pval.tmp)
  }
}
print(paste("Mann-Whitney U test p-value: ", mean(pvalues, na.rm=TRUE)))

nb_CDS_exons <- 3901
nb_orphan_denovo_exons <- 2871 # over 1862 orphan dn CDS
nb_iORF_exons <- 1000

nb_CDS_exons_cov <- length(unique(df_CDS_1000_exons$V1))
nb_CDS_exons_uncov <- nb_CDS_exons - nb_CDS_exons_cov
nb_CDS_exons_cov_90 <- length(unique(df_CDS_1000_exons[df_CDS_1000_exons$V2 >= 90, ]$V1))
nb_CDS_exons_cov_95 <- length(unique(df_CDS_1000_exons[df_CDS_1000_exons$V2 >= 95, ]$V1))
nb_CDS_exons_all_neighbors <- sum(table(df_CDS_1000_exons$V1))/nb_CDS_exons
nb_CDS_exons_all_cov_90_neighbors <- sum(table(df_CDS_1000_exons$V1[df_CDS_1000_exons$V2 >= 90]))/nb_CDS_exons
nb_CDS_exons_cov_neighbor <- mean(table(df_CDS_1000_exons$V1))
nb_CDS_exons_cov_90_neighbor <- mean(table(df_CDS_1000_exons$V1[df_CDS_1000_exons$V2 >= 90]))

nb_orphan_denovo_exons_cov <- length(unique(df_oprhan_denovo_exons$V1))
nb_orphan_denovo_exons_uncov <- nb_orphan_denovo_exons - nb_orphan_denovo_exons_cov
nb_orphan_denovo_exons_cov_90 <- length(unique(df_oprhan_denovo_exons[df_oprhan_denovo_exons$V2 >= 90, ]$V1))
nb_orphan_denovo_exons_cov_95 <- length(unique(df_oprhan_denovo_exons[df_oprhan_denovo_exons$V2 >= 95, ]$V1))
nb_orphan_denovo_exon_all_neighbors <- sum(table(df_oprhan_denovo_exons$V1))/nb_orphan_denovo_exons
nb_orphan_denovo_exon_all_cov_90_neighbors <- sum(table(df_oprhan_denovo_exons$V1[df_oprhan_denovo_exons$V2 >= 90]))/nb_orphan_denovo_exons
nb_orphan_denovo_exons_cov_neighbor <- mean(table(df_oprhan_denovo_exons$V1))
nb_orphan_denovo_exons_cov_90_neighbor <- mean(table(df_oprhan_denovo_exons$V1[df_oprhan_denovo_exons$V2 >= 90]))

nb_iORF_exons_cov <- length(unique(df_iORF_1000_exons$V1))
nb_iORF_exons_uncov <- nb_iORF_exons - nb_iORF_exons_cov
nb_iORF_exons_cov_90 <- length(unique(df_iORF_1000_exons[df_iORF_1000_exons$V2 >= 90, ]$V1))
nb_iORF_exons_cov_95 <- length(unique(df_iORF_1000_exons[df_iORF_1000_exons$V2 >= 95, ]$V1))
nb_iORF_exons_all_neighbors <- sum(table(df_iORF_1000_exons$V1))/nb_iORF_exons
nb_iORF_exons_all_cov_90_neighbors <- sum(table(df_iORF_1000_exons$V1[df_iORF_1000_exons$V2 >= 90]))/nb_iORF_exons
nb_iORF_exons_cov_neighbor <- mean(table(df_iORF_1000_exons$V1))
nb_iORF_exons_cov_90_neighbor <- mean(table(df_iORF_1000_exons$V1[df_iORF_1000_exons$V2 >= 90]))

exon_stats_df <- data.frame( type = c("CDS","orphan_denovo","iORF"),
                             nb_seq = c(3901, 1862, 1000),
                             nb_exons = c(nb_CDS_exons, nb_orphan_denovo_exons, nb_iORF_exons),
                             nb_exons_cov = c(nb_CDS_exons_cov, nb_orphan_denovo_exons_cov, nb_iORF_exons_cov),
                             nb_exons_uncov = c(nb_CDS_exons_uncov, nb_orphan_denovo_exons_uncov, nb_iORF_exons_uncov),
                             nb_exons_cov_90 = c(nb_CDS_exons_cov_90, nb_orphan_denovo_exons_cov_90, nb_iORF_exons_cov_90),
                            nb_exons_cov_95 = c(nb_CDS_exons_cov_95, nb_orphan_denovo_exons_cov_95, nb_iORF_exons_cov_95),
                             perc_exons_cov = c(round(nb_CDS_exons_cov/nb_CDS_exons*100), round(nb_orphan_denovo_exons_cov/nb_orphan_denovo_exons*100), round(nb_iORF_exons_cov/nb_iORF_exons*100)),
                             perc_exons_uncov = c(round(nb_CDS_exons_uncov/nb_CDS_exons*100), round(nb_orphan_denovo_exons_uncov/nb_orphan_denovo_exons*100), round(nb_iORF_exons_uncov/nb_iORF_exons*100)),
                             perc_exons_cov_90 = c(round(nb_CDS_exons_cov_90/nb_CDS_exons*100), round(nb_orphan_denovo_exons_cov_90/nb_orphan_denovo_exons*100), round(nb_iORF_exons_cov_90/nb_iORF_exons*100)),
                            perc_exons_cov_95 = c(round(nb_CDS_exons_cov_95/nb_CDS_exons*100), round(nb_orphan_denovo_exons_cov_95/nb_orphan_denovo_exons*100), round(nb_iORF_exons_cov_95/nb_iORF_exons*100)),
                            all_mean_neighbor = c(nb_CDS_exons_all_neighbors, nb_orphan_denovo_exon_all_neighbors, nb_iORF_exons_all_neighbors),
                            all_cov90_mean_neighbor = c(nb_CDS_exons_all_cov_90_neighbors, nb_orphan_denovo_exon_all_cov_90_neighbors, nb_iORF_exons_all_cov_90_neighbors),
                            cov_mean_neighbor = c(nb_CDS_exons_cov_neighbor, nb_orphan_denovo_exons_cov_neighbor, nb_iORF_exons_cov_neighbor),
                            cov_mean_90_neighbor = c(nb_CDS_exons_cov_90_neighbor, nb_orphan_denovo_exons_cov_90_neighbor, nb_iORF_exons_cov_90_neighbor),
                             stringsAsFactors = FALSE)
write.table(exon_stats_df, "data/Fig4B/exon_stats.tsv", sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE)


### ABSOLUTE
nb_CDS_exons <- 3901*11
nb_orphan_denovo_exons <- 2871*11
nb_iORF_exons <- 1000*11

nb_CDS_exons_cov <- length(df_CDS_1000_exons$V1)
nb_CDS_exons_uncov <- nb_CDS_exons - nb_CDS_exons_cov
nb_CDS_exons_cov_90 <- length(df_CDS_1000_exons[df_CDS_1000_exons$V2 >= 90, ]$V1)
nb_CDS_exons_cov_95 <- length(df_CDS_1000_exons[df_CDS_1000_exons$V2 >= 95, ]$V1)
nb_CDS_exons_all_neighbors <- sum(table(df_CDS_1000_exons$V1))/(nb_CDS_exons/11)
nb_CDS_exons_all_cov_90_neighbors <- sum(table(df_CDS_1000_exons$V1[df_CDS_1000_exons$V2 >= 90]))/(nb_CDS_exons/11)
nb_CDS_exons_cov_neighbor <- mean(table(df_CDS_1000_exons$V1))
nb_CDS_exons_cov_90_neighbor <- mean(table(df_CDS_1000_exons$V1[df_CDS_1000_exons$V2 >= 90]))

nb_orphan_denovo_exons_cov <- length(df_oprhan_denovo_exons$V1)
nb_orphan_denovo_exons_uncov <- nb_orphan_denovo_exons - nb_orphan_denovo_exons_cov
nb_orphan_denovo_exons_cov_90 <- length(df_oprhan_denovo_exons[df_oprhan_denovo_exons$V2 >= 90, ]$V1)
nb_orphan_denovo_exons_cov_95 <- length(df_oprhan_denovo_exons[df_oprhan_denovo_exons$V2 >= 95, ]$V1)
nb_orphan_denovo_exon_all_neighbors <- sum(table(df_oprhan_denovo_exons$V1))/(nb_orphan_denovo_exons/11)
nb_orphan_denovo_exon_all_cov_90_neighbors <- sum(table(df_oprhan_denovo_exons$V1[df_oprhan_denovo_exons$V2 >= 90]))/(nb_orphan_denovo_exons/11)
nb_orphan_denovo_exons_cov_neighbor <- mean(table(df_oprhan_denovo_exons$V1))
nb_orphan_denovo_exons_cov_90_neighbor <- mean(table(df_oprhan_denovo_exons$V1[df_oprhan_denovo_exons$V2 >= 90]))

nb_iORF_exons_cov <- length(df_iORF_1000_exons$V1)
nb_iORF_exons_uncov <- nb_iORF_exons - nb_iORF_exons_cov
nb_iORF_exons_cov_90 <- length(df_iORF_1000_exons[df_iORF_1000_exons$V2 >= 90, ]$V1)
nb_iORF_exons_cov_95 <- length(df_iORF_1000_exons[df_iORF_1000_exons$V2 >= 95, ]$V1)
nb_iORF_exons_all_neighbors <- sum(table(df_iORF_1000_exons$V1))/(nb_iORF_exons/11)
nb_iORF_exons_all_cov_90_neighbors <- sum(table(df_iORF_1000_exons$V1[df_iORF_1000_exons$V2 >= 90]))/(nb_iORF_exons/11)
nb_iORF_exons_cov_neighbor <- mean(table(df_iORF_1000_exons$V1))
nb_iORF_exons_cov_90_neighbor <- mean(table(df_iORF_1000_exons$V1[df_iORF_1000_exons$V2 >= 90]))

exon_stats_df <- data.frame( type = c("CDS","orphan_denovo","iORF"),
                             nb_seq = c(3901, 1862, 1000),
                             nb_exons = c(nb_CDS_exons, nb_orphan_denovo_exons, nb_iORF_exons),
                             nb_exons_cov = c(nb_CDS_exons_cov, nb_orphan_denovo_exons_cov, nb_iORF_exons_cov),
                             nb_exons_uncov = c(nb_CDS_exons_uncov, nb_orphan_denovo_exons_uncov, nb_iORF_exons_uncov),
                             nb_exons_cov_90 = c(nb_CDS_exons_cov_90, nb_orphan_denovo_exons_cov_90, nb_iORF_exons_cov_90),
                            nb_exons_cov_95 = c(nb_CDS_exons_cov_95, nb_orphan_denovo_exons_cov_95, nb_iORF_exons_cov_95),
                             perc_exons_cov = c(round(nb_CDS_exons_cov/nb_CDS_exons*100), round(nb_orphan_denovo_exons_cov/nb_orphan_denovo_exons*100), round(nb_iORF_exons_cov/nb_iORF_exons*100)),
                             perc_exons_uncov = c(round(nb_CDS_exons_uncov/nb_CDS_exons*100), round(nb_orphan_denovo_exons_uncov/nb_orphan_denovo_exons*100), round(nb_iORF_exons_uncov/nb_iORF_exons*100)),
                             perc_exons_cov_90 = c(round(nb_CDS_exons_cov_90/nb_CDS_exons*100), round(nb_orphan_denovo_exons_cov_90/nb_orphan_denovo_exons*100), round(nb_iORF_exons_cov_90/nb_iORF_exons*100)),
                            perc_exons_cov_95 = c(round(nb_CDS_exons_cov_95/nb_CDS_exons*100), round(nb_orphan_denovo_exons_cov_95/nb_orphan_denovo_exons*100), round(nb_iORF_exons_cov_95/nb_iORF_exons*100)),
                            all_mean_neighbor = c(nb_CDS_exons_all_neighbors, nb_orphan_denovo_exon_all_neighbors, nb_iORF_exons_all_neighbors),
                            all_cov90_mean_neighbor = c(nb_CDS_exons_all_cov_90_neighbors, nb_orphan_denovo_exon_all_cov_90_neighbors, nb_iORF_exons_all_cov_90_neighbors),
                            cov_mean_neighbor = c(nb_CDS_exons_cov_neighbor, nb_orphan_denovo_exons_cov_neighbor, nb_iORF_exons_cov_neighbor),
                            cov_mean_90_neighbor = c(nb_CDS_exons_cov_90_neighbor, nb_orphan_denovo_exons_cov_90_neighbor, nb_iORF_exons_cov_90_neighbor),
                            stringsAsFactors = FALSE)
write.table(exon_stats_df, "data/Fig4B/exon_stats_asb.tsv", sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE)


# Use the discretize function to discretize the data
# df_CDS_1000_exons$V2 <- discretize(df_CDS_1000_exons$V2, method = "frequency", breaks = 20)

plot_CDS <- ggplot(data = df_CDS_1000_exons, aes(x = V2)) + 
  geom_histogram(binwidth = 5, fill = "blue", alpha = 0.5) +
  labs(x = "A coverage", y = "Number of exons", title = "CDS 1000") +
  theme_bw() +
  theme(plot.title = element_text(hjust = 0.5))

plot_orphan_denovo <- ggplot(data = df_oprhan_denovo_exons, aes(x = V2)) +
    geom_histogram(binwidth = 5, fill = "blue", alpha = 0.5) +
    labs(x = "A coverage", y = "Number of exons", title = "Orphan denovo") +
    theme_bw() +
    theme(plot.title = element_text(hjust = 0.5))

plot_iORF_1000 <- ggplot(data = df_iORF_1000_exons, aes(x = V2)) +
    geom_histogram(binwidth = 5, fill = "blue", alpha = 0.5) +
    labs(x = "A coverage", y = "Number of exons", title = "iORF 1000") +
    theme_bw() +
    theme(plot.title = element_text(hjust = 0.5))

pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig4B.pdf"))

plot_grid(plot_CDS, plot_orphan_denovo, plot_iORF_1000, ncol = 3, labels = c("A", "B", "C"), label_size = 12)

dev.off()


#### Fig 4C ####
# This script generates the features boxplots of Osat (Fig2C).

p_size <- get_exclusive_plot(NAME = "OSAT",
                   feature = "size",
                   groups = c("IORF","NCOS_orphan","NCOS_non_orphan","nonTRG"),
                   comps = list(c("IORF","NCOS_orphan"),
                                c("NCOS_orphan","NCOS_non_orphan"),
                                c("NCOS_non_orphan","nonTRG")),
                   yp=c(350,400,450))+
                   ylab("Length")+
                   scale_x_discrete(labels = c("IORF" = "NC ORFs", "NCOS_orphan" = "de novo orphan", "NCOS_non_orphan" = "de novo non-orphan", "nonTRG" = "non-TRG"))+
                   theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 18))

# p_size
p_gan <- get_exclusive_plot(NAME = "OSAT",
                   feature = "GAN",
                   groups = c("IORF","NCOS_orphan","NCOS_non_orphan","nonTRG"),
                   comps = list(c("IORF","NCOS_orphan"),
                                c("NCOS_orphan","NCOS_non_orphan"),
                                c("NCOS_non_orphan","nonTRG")),
                   yp=c(0.15,0.175,0.20))+
                   ylab("Fraction of negatively\ncharged residues")+
                   scale_x_discrete(labels = c("IORF" = "NC ORFs", "NCOS_orphan" = "de novo orphan", "NCOS_non_orphan" = "de novo non-orphan", "nonTRG" = "non-TRG"))+
                   theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 18))
# p_gan
p_distance <- get_exclusive_plot(NAME = "OSAT",
                   feature = "gene_dist",
                   groups = c("NCOS_orphan","NCOS_non_orphan","nonTRG"),
                   comps = list(c("NCOS_orphan","NCOS_non_orphan"),
                                c("NCOS_non_orphan","nonTRG")),
                   yp=c(600,700))+
                   ylab("Distance to closest\ngene (nucl)")+
                   scale_x_discrete(labels = c("IORF" = "NC ORFs", "NCOS_orphan" = "de novo orphan", "NCOS_non_orphan" = "de novo non-orphan", "nonTRG" = "non-TRG"))+
                   theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 18))
# p_distance
p_dNdS_p <- get_exclusive_plot(NAME = "OSAT",
                   feature = "p_neg",
                   groups = c("NCOS_non_orphan","nonTRG"),
                   comps = list(c("NCOS_non_orphan","nonTRG")),
                   yp=c(0.95))+
                   ylab("Fraction of residues\nunder neg. selection")+
                   scale_x_discrete(labels = c("IORF" = "NC ORFs", "NCOS_orphan" = "de novo orphan", "NCOS_non_orphan" = "de novo non-orphan", "nonTRG" = "non-TRG"))+
                   theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 18))
# p_dNdS_p

plots <- list(p_size,p_gan,p_distance,p_dNdS_p)
plots <- lapply(plots, function(x) x + theme(legend.position = "none",
                                             axis.title.x = element_blank(),
                                             axis.text.x = element_text(size = 5))
                                     + ggtitle(NULL))
features_grid <- plot_grid(plotlist = plots, ncol = 4)
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig4C.pdf"), height = 2)
features_grid
dev.off()

# h_spacer <- 3

# p_title <- ggdraw()+draw_label("O. sativa features", size = 10, x = 0.5, y = 0.5)
# fig2_grid <- plot_grid(p_title,
#                        NULL,
#                        HM_grid, # from trees_HM_CDS_iORF.R
#                        NULL,
#                        features_grid, nrow = 5, rel_heights = c(1,h_spacer,20,h_spacer,10))

# pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig2.pdf"))
# fig2_grid
# dev.off()



##############
#### Fig5 ####
##############


# This script generates Fig5 A and B.
text_size <- 16
#### Fig 5A ####
df_a <- read.csv("data/Fig5/Fig5A.csv")
df_a$status <- factor(df_a$status, levels = c("denovo","trg"), labels = c("de novo genes","TRG"))

a <- ggplot(data = df_a, aes(x=lines, y=nb, fill = status)) +
  geom_col(position="stack") + # This is equivalent to geom_bar(stat="identity") but more explicit
  scale_fill_manual(values = c("#bebebe","#979eb7")) +
  geom_line(aes(x=lines, y=rep(aggregate(nb ~ lines, data = df_a, sum)[,"nb"], each = 2)))+
  theme_minimal() +
  labs(x = "Number of lines", y = "Counts")+
  theme(
    legend.title = element_blank(),
    title = element_text(size = text_size),
    axis.text.y = element_text(size = text_size),
    axis.ticks.y = element_blank(),
    axis.line.y = element_blank(),
    axis.text.x = element_text(size = text_size),
    axis.ticks.x = element_blank(),
    axis.line.x = element_blank()
  )+
  ggtitle(expression("TRGs and " * italic("de novo") * " genes shared by lines"))+
  scale_x_discrete(limits = factor(unique(df_a$lines)))
a

#### Fig 5B ####
df_b <- read.csv("data/Fig5/Fig5B.csv")
df_b$status <- factor(df_b$status, levels = c("denovo"), labels = c("neORFs"))

b <- ggplot(data = df_b, aes(x=lines, y=nb, fill = status)) +
  geom_col(position="stack") + # This is equivalent to geom_bar(stat="identity") but more explicit
  scale_fill_manual(values = c("#cfa9a8")) +
  geom_line(aes(x=lines, y=nb))+
  theme_minimal() +
  labs(x = "Number of lines", y = "Counts")+
  theme(
    legend.title = element_blank(),
    title = element_text(size = text_size),
    axis.text.y = element_text(size = text_size),
    axis.ticks.y = element_blank(),
    axis.line.y = element_blank(),
    axis.text.x = element_text(size = text_size),
    axis.ticks.x = element_blank(),
    axis.line.x = element_blank()
  )+
  ggtitle("neORFs shared by lines (Grandchamp 2023)")+
  scale_x_discrete(limits = factor(df_b$lines)) # Ensure every x-axis value is displayed
b


# This script generates Fig5 C and D.

main_df <- read.table("data/DMEL_main_dataframe.tsv", header = TRUE)
IORF_NCOS_df <- main_df[(main_df$NCOS == "TRUE" & ! is.na(main_df$NCOS)) | main_df$seq_type == "IORF", ]
IORF_NCOS_df$seq_type[IORF_NCOS_df$seq_type == "CDS"] <- "NCOS"
IORF_NCOS_df <- IORF_NCOS_df[ ,c("seq_name","seq_type","size","GAN","gene_dist")]

# Gene distance
IORF_gene_dist <- read.table("data/Fig5/Dmel_iORF_1000_closest_gene.tsv", header = TRUE)
colnames(IORF_gene_dist) <- c("seq_name","null","gene_dist")
IORF_gene_dist <- IORF_gene_dist[ ,c("seq_name","gene_dist")]
IORF_NCOS_df <- merge(IORF_NCOS_df, IORF_gene_dist, by="seq_name", all.x=TRUE)
IORF_NCOS_df$gene_dist.x[is.na(IORF_NCOS_df$gene_dist.x)] <- IORF_NCOS_df$gene_dist.y[is.na(IORF_NCOS_df$gene_dist.x)]
IORF_NCOS_df$gene_dist <- IORF_NCOS_df$gene_dist.x
IORF_NCOS_df <- IORF_NCOS_df[ ,c("seq_name","seq_type","size","GAN","gene_dist")]

### NEO-TRANSCRIPTS ###
neot_size <- read.table("data/Fig5/denovo_transcripts_spliced_size.tsv", header = FALSE)
colnames(neot_size) <- c("seq_name","size")
neot_cc <- read.csv("data/Fig5/denovo_transcripts_spliced_cc.csv")
colnames(neot_cc)[1] <- "seq_name"
neot_cc[,-1] <- neot_cc[,-1]/rowSums(neot_cc[,-1])
neot_cc$GAN <- rowSums(neot_cc[ ,c("GAT","GAC","GAA","GAG")])
neot_GAN <- data.frame(seq_name = neot_cc$seq_name, GAN = neot_cc$GAN)
neot_gene_dist <- read.csv("data/Fig5/distance_rsts.csv")
colnames(neot_gene_dist) <- c("seq_name","gene_dist")
neot_df <- merge(neot_size, neot_GAN, by="seq_name")
neot_df <- merge(neot_df, neot_gene_dist, by="seq_name")
neot_df$seq_type <- "neoTrans"

df <- rbind(IORF_NCOS_df,
            neot_df)

df$log_gene_dist <- log(df$gene_dist)

# Test size
dim(df)
dim(df[df$seq_type == "IORF", ])
dim(df[df$seq_type == "NCOS", ])
dim(df[df$seq_type == "neoTrans", ])

MWW <- function(group1, group2, alternative){
  pvalues = NULL
  if (length(group1) > 500 || length(group2) > 500){
    for (i in 1:100){
      if (length(group1) > 500){
        s1 = sample(group1, size = 500)
      } else {
        s1 = group1
      }
      if (length(group2) > 500){
        s2 = sample(group2, size = 500)
      } else {
        s2 = group2
      }
      pval.tmp = wilcox.test(s1, s2, alternative = "greater", exact = FALSE)$p.value
      pvalues = c(pvalues,pval.tmp)
    }
  }
  pv = mean(pvalues, na.rm=TRUE)
  return(pv)
}
neot_gt_iorf <- MWW(df[df$seq_type == "neoTrans", ]$size, df[df$seq_type == "IORF", ]$size, "greater")
print(paste("Mann-Whitney U test p-value: ", neot_gt_iorf))
ncos_gt_iorf <- MWW(df[df$seq_type == "NCOS", ]$size, df[df$seq_type == "IORF", ]$size, "greater")
print(paste("Mann-Whitney U test p-value: ", ncos_gt_iorf))
ncos_gt_iorf <- MWW(df[df$seq_type == "NCOS", ]$size, df[df$seq_type == "neoTrans", ]$size, "greater")
print(paste("Mann-Whitney U test p-value: ", ncos_gt_iorf))

melted_df <- melt(df, measure.vars = c("seq_type"))
melted_df$variable <- melted_df$value
melted_df$variable <- factor(melted_df$variable, levels = c("IORF", "neoTrans", "NCOS"), ordered = TRUE)


# Create separate data frames for the points and violins
points_data <- melted_df
violins_data <- melted_df

# Define colors
colors <- c("darkorchid","lightgrey","#fad7ad","darkorange","grey","#fad2a0","#fcc583","#fda232","#cea8a7")
names(colors) <- c("IORF","NCOS_orphan","NCOS_non_orphan","nonTRG","NCOS","NCO","NC","TRG","neoTrans")
colors_to_use <- colors[levels(points_data$variable)]


#### Fig5C ####
c <- ggplot(data=melted_df, aes(x=variable, y=size))+
  geom_violin(data = violins_data, aes(x = variable, y = size, colour=variable), alpha = 0.2)+
  geom_sina(data = points_data, aes(x = variable, y = size, fill=variable, colour = variable), shape = 21, size = 0.125)+
  scale_fill_manual(values = colors_to_use) +  # Fill color for points
  scale_colour_manual(values = colors_to_use) +  # Same border color as fill for points
  stat_summary(data = points_data, aes(x = variable, y = size), fun = "median",
                    geom = "crossbar",
                    width = 0.5,  # Reduce the width
                    lwd = 0.125)+
  stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                comps = list(c("IORF", "neoTrans"),
                                                              c("neoTrans", "NCOS"),
                                                              c("IORF", "NCOS")),
                                                col.x = "variable",
                                                col.y = "size"), tip.length = 0, y.position = c(200,300,400)) + theme(legend.position = "none")+
  # coord_cartesian(ylim = c(0,1))+
  theme_minimal()+
  theme(
    axis.title.x = element_blank(),
    axis.text.x = element_text(size = text_size),
    axis.text.y = element_text(size = text_size),
    axis.title.y = element_text(size = text_size),
    legend.position = "none")+
  ylab("Length")+
  scale_x_discrete(labels = c("IORF" = "NC ORFs", "neoTrans" = "neORFs", "NCOS" = "de novo genes"))



#### Fig5D ####
d <- ggplot(data=melted_df, aes(x=variable, y=GAN))+
  geom_violin(data = violins_data, aes(x = variable, y = GAN, colour=variable), alpha = 0.2)+
  geom_sina(data = points_data, aes(x = variable, y = GAN, fill=variable, colour = variable), shape = 21, size = 0.125)+
  scale_fill_manual(values = colors_to_use) +  # Fill color for points
  scale_colour_manual(values = colors_to_use) +  # Same border color as fill for points
  stat_summary(data = points_data, aes(x = variable, y = GAN), fun = "median",
                    geom = "crossbar",
                    width = 0.5,  # Reduce the width
                    lwd = 0.125)+
  stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                comps = list(c("IORF", "neoTrans"),
                                                              c("neoTrans", "NCOS"),
                                                              c("IORF", "NCOS")),
                                                col.x = "variable",
                                                col.y = "GAN"), tip.length = 0, y.position = c(0.10,0.15,0.20)) + theme(legend.position = "none")+
  # coord_cartesian(ylim = c(0,1))+
  theme_minimal()+
  theme(
    axis.title.x = element_blank(),
    axis.text.x = element_text(size = text_size),
    axis.text.y = element_text(size = text_size),
    axis.title.y = element_text(size = text_size),
    legend.position = "none")+
  ylab("Fraction of negatively\ncharged residues")+
  scale_x_discrete(labels = c("IORF" = "NC ORFs", "neoTrans" = "neORFs", "NCOS" = "de novo genes"))


  #### Fig5E ####
  e <- ggplot(data=melted_df, aes(x=variable, y=log_gene_dist))+
  geom_violin(data = violins_data, aes(x = variable, y = log_gene_dist, colour=variable), alpha = 0.2)+
  geom_sina(data = points_data, aes(x = variable, y = log_gene_dist, fill=variable, colour = variable), shape = 21, size = 0.125)+
  scale_fill_manual(values = colors_to_use) +  # Fill color for points
  scale_colour_manual(values = colors_to_use) +  # Same border color as fill for points
  stat_summary(data = points_data, aes(x = variable, y = log_gene_dist), fun = "median",
                    geom = "crossbar",
                    width = 0.5,  # Reduce the width
                    lwd = 0.125)+
  stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
                                                comps = list(c("IORF", "neoTrans"),
                                                              c("neoTrans", "NCOS"),
                                                              c("IORF", "NCOS")),
                                                col.x = "variable",
                                                col.y = "gene_dist"), tip.length = 0, y.position = c(1,2,3)) + theme(legend.position = "none")+
  # coord_cartesian(ylim = c(0,1))+
  theme_minimal()+
  ylab("Distance to\nclosest gene (log(bp))")+
  theme(
    axis.title.x = element_blank(),
    axis.text.x = element_text(size = text_size),
    axis.text.y = element_text(size = text_size),
    axis.title.y = element_text(size = text_size),
    legend.position = "none")+
  scale_x_discrete(labels = c("IORF" = "NC ORFs", "neoTrans" = "neORFs", "NCOS" = "de novo genes"))


#### GRID ####
pdf(file="Fig5.pdf", width = 15, height = 10)
plot_grid(
  plot_grid(a, b,    ncol=2, labels = c("a)", "b)"),     label_size = 20),
  NULL,
  plot_grid(c, d, e, ncol=3, labels = c("c)", "d)", "e)"), label_size = 20, label_y = 1.05),
  nrow=3, rel_heights = c(1,0.1,1)
)
dev.off()


##############
#### Fig6 ####
##############
# This script generates the features boxplots of the different species (Fig3).

# Function that remove the legend and the axis.
remove_legend_and_axis <- function(plot){
  plot <- plot + theme(legend.position = "none",
                        axis.title.x = element_blank(),
                        axis.text.x = element_blank(),
                        axis.ticks.x = element_blank(),
                        axis.title.y = element_blank(),
                        axis.text.y = element_blank(),
                        axis.ticks.y = element_blank())+
                  ggtitle(NULL)
  return(plot)
}
plots_to_one_line <- function(plot_list, y_title_text = "y_title"){
  
  # Get the y axis of the first plot.
  y_title_text <- plot_list[[1]]$labels$y
  y_title <- ggdraw()+draw_label(y_title_text, x = 0.2, y = 0.5, size = 6, angle = 90)
  y_axis <- get_y_axis(plot_list[[1]]+ theme(axis.text.y = element_text(size = 6)))

  # For each plot in plot_list, remove the legend and the axis.
  plot_list_formatted <- lapply(plot_list, remove_legend_and_axis)

  # Get the same order as names_df$NAME
  plot_list_formatted <- plot_list_formatted[names_df$NAME]

  # Unique list with the y title, the y axis and the plots.
  plotlist <- c(list(y_title, y_axis), plot_list_formatted)

  # All genomes on one line :
  oneline_plot <- plot_grid(plotlist = plotlist, nrow = 1, rel_widths = c(2,1,rep(5,length(plot_list_formatted))))

  return(oneline_plot)
}

#### SIZES ####
size_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"size plot..."))
  comparisons <- list(c("NCOS","NCO"),
                      c("NCOS","NC"),
                      c("NCOS","TRG"),
                      c("NCOS","nonTRG"))
  size_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                          feature = "size",
                                          groups = c("NCOS","NCO","NC","TRG","nonTRG"),
                                          comps = comparisons,
                                          yp=seq(200, 500, length.out = length(comparisons)))+
                                          ylab("Length")
}

#### GAN ####
GAN_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"GAN plot..."))
  GAN_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                          feature = "GAN",
                                          groups = c("NCOS","NCO","NC","TRG","nonTRG"),
                                          comps = comparisons,
                                          yp=seq(0.125, 0.3, length.out = length(comparisons)))+
                                          ylab("Fraction of negatively\ncharged residues")
}

#### gene_dist ####
gene_dist_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"gene_dist plot..."))
  gene_dist_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                                feature = "gene_dist",
                                                groups = c("NCOS","NCO","NC","TRG","nonTRG"),
                                                comps = comparisons,
                                                yp=seq(350, 1000, length.out = length(comparisons)))+
                                                ylab("Distance to closest\ngene (nucl)")
}

### dNdS_p ####
dNdS_p_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"dNdS_p plot..."))
  dNdS_p_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                            feature = "p_neg",
                                            groups = c("NCOS","NCO","NC","TRG","nonTRG"),
                                            comps = comparisons,
                                            yp=seq(0.05, 0.5, length.out = length(comparisons)))+
                                            ylab("Fraction of residues\nunder neg. selection")
}

#### age ####
age_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"age plot..."))
  comparisons <- list(c("NCOS","NCO"),
                    c("NCOS","NC"),
                    c("NCOS","TRG"))
  age_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                          feature = "age",
                                          groups = c("NCOS","NCO","NC","TRG"),
                                          comps = comparisons,
                                          yp=seq(0.15, 0.75, length.out = length(comparisons)))+
                                          ylab("Phylogenetic distance")
}

###############


sizes <- plots_to_one_line(size_plots)
GANs <- plots_to_one_line(GAN_plots)
gene_dists <- plots_to_one_line(gene_dist_plots)
dNdS_ps <- plots_to_one_line(dNdS_p_plots)
ages <- plots_to_one_line(age_plots)
names_plots <- plot_grid(plotlist = c( list(NULL,NULL), lapply(names_df$NAME, function(x) ggdraw()+draw_label(x, x = 0.5, y = 0.5, size = 10))),
                         nrow = 1,
                         rel_widths = c(2,1,rep(5,length(names_df$NAME))))
# names_plots <- plot_grid(plotlist = c( list(NULL,NULL), lapply(c("SCER","OSAT","HSAP","DMEL"), function(x) ggdraw()+draw_label(x, x = 0.5, y = 0.5, size = 10))),
#                          nrow = 1,
#                          rel_widths = c(2,1,rep(5,length(c("SCER","OSAT","HSAP","DMEL")))))
horizontal_spacer_size <- 1
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig6.pdf"))

plot_grid(names_plots,
          NULL,
          sizes,
          NULL,
          GANs,
          NULL,
          gene_dists,
          NULL,
          dNdS_ps,
          NULL,
          ages,
          NULL,
          nrow = 12,
          rel_heights = c(1,horizontal_spacer_size,rep(c(5,horizontal_spacer_size),5)))

dev.off()




##############
#### Fig7 ####
##############

#### Fig7A ####
a_df <- read.csv("data/Fig7/window_1_to_10.csv")
a_df_no_name <- a_df[,-which(names(a_df) %in% c("NAME"))]
a_df_no_name$max <- do.call(pmax, c(a_df_no_name, list(na.rm=TRUE)))
a_df[,-which(names(a_df) %in% c("NAME"))] <- a_df[,-which(names(a_df) %in% c("NAME"))]/a_df_no_name$max

mean(a_df$win1)
mean(a_df$win4)

melted_df <- melt(a_df, id.vars = c("NAME"))

# Remove "win" from each element in the variable column
melted_df$variable <- sub("win", "", melted_df$variable)

# Convert the variable column to numeric
melted_df$variable <- as.numeric(as.character(melted_df$variable))

a <- ggplot(data = melted_df, aes(x = variable, y= value))+
  geom_point(aes(color= NAME, group=NAME))+
  geom_line(aes(color= NAME, group=NAME))+
  geom_smooth(method = "loess", se = FALSE, color = rgb(0.5, 0.5, 0.5, alpha = 0.2), linewidth = 10)+
  theme_minimal()+
  xlab("Window size")+
  ylab("Fraction of detected de novo CDSs")+
  scale_x_continuous(breaks = melted_df$variable)+
  theme(panel.grid.minor = element_blank(),
        axis.title.y = element_text(hjust = -0.5))


#### Fig7B ####
b_df <- read.csv("data/Fig7/window6_anchors_1_to_6.csv")
b_df_no_name <- b_df[,-which(names(b_df) %in% c("NAME"))]
b_df_no_name$max <- do.call(pmax, c(b_df_no_name, list(na.rm=TRUE)))
b_df[,-which(names(b_df) %in% c("NAME"))] <- b_df[,-which(names(b_df) %in% c("NAME"))]/b_df_no_name$max

melted_df <- melt(b_df, id.vars = c("NAME"))

# Remove "win" from each element in the variable column
melted_df$variable <- sub("X", "", melted_df$variable)
melted_df$variable <- sub("anch", "", melted_df$variable)

# Convert the variable column to numeric
melted_df$variable <- as.numeric(as.character(melted_df$variable))

b <- ggplot(data = melted_df, aes(x = variable, y= value))+
  geom_point(aes(color= NAME, group=NAME))+
  geom_line(aes(color= NAME, group=NAME))+
  geom_smooth(method = "loess", se = FALSE, color = rgb(0.5, 0.5, 0.5, alpha = 0.2), linewidth = 10)+
  theme_minimal()+
  xlab("Number of anchor pairs")+
  ylab("Fraction of detected de novo CDSs")+
  scale_x_continuous(breaks = melted_df$variable)+
  theme(panel.grid.minor = element_blank(),
        axis.title.y = element_text(hjust = -0.5))


#### Fig7C ####
c_df <- read.csv("data/Fig7/num_outrgroups_table.csv")
# All colnames minus "NAME"
c_df[,colnames(c_df)[-1]] <- c_df[,colnames(c_df)[-1]] / c_df$num_out_1

melted_df <- melt(c_df, id.vars = c("NAME"))
melted_df$variable <- as.numeric(sub("num_out_", "", melted_df$variable))
melted_df <- melted_df[!is.na(melted_df$value),]

c <- ggplot(melted_df, aes(x = variable, y = value, color = NAME)) +
  geom_point(aes(color= NAME, group=NAME))+
  geom_line(aes(color= NAME, group=NAME))+
  geom_smooth(method = "loess", se = FALSE, color = rgb(0.5, 0.5, 0.5, alpha = 0.2), linewidth = 10)+
  theme_minimal() +
  labs(x = "Noncoding outgroups required", y = "Fraction of detected de novo CDSs") +
  scale_x_continuous(breaks = melted_df$variable)+
  theme(panel.grid.minor = element_blank(),
        axis.title.y = element_text(hjust = -0.5))
  


#### GRID ####
a <- a + scale_color_discrete(labels = c("A. thaliana","C. elegans","D. melanogaster","H. sapiens","M. musculus","O. sativa","S. cerevisiae"))
a <- a + theme(legend.text = element_text(face = "italic"))
a <- a + theme(legend.margin = margin(10, 10, 10, 10), legend.title = element_blank())

legend <- get_legend(a)
axis_title_size <- 8
# pm <- unit(c(5.1, 4.1, 4.1, 2.1), "pt") # default
pm <- unit(c(5.1, 4.1, 4.1, 10.1), "pt")
a <- a+theme(legend.position = "none", axis.title.x = element_text(size = axis_title_size), axis.title.y = element_text(size = axis_title_size),plot.margin = pm)
b <- b+theme(legend.position = "none", axis.title.x = element_text(size = axis_title_size), axis.title.y = element_text(size = axis_title_size),plot.margin = pm)
c <- c+theme(legend.position = "none", axis.title.x = element_text(size = axis_title_size), axis.title.y = element_text(size = axis_title_size),plot.margin = pm)

pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "Fig7.pdf"), width = 6, height = 5)
plot_grid(c,legend, a, b, ncol=2, nrow=2, labels = c("a)","","b)","c)"), rel_widths = c(1,1))
dev.off()


################
#### Fig S1 ####
################

plots <- list()
correlation_df <- data.frame(NAME = character(), rho = numeric(), p_value = numeric())
header_height <- list()
header_height[["SCER"]] <- 2
header_height[["ATHA"]] <- 2.5
header_height[["OSAT"]] <- 1
header_height[["CELE"]] <- 0.5
header_height[["DMEL"]] <- 0.5
header_height[["MMUS"]] <- 1
header_height[["HSAP"]] <- 2

# for (NAME in c("SCER","ATHA","OSAT","CELE","DMEL","MMUS")){ # Waiting for HSAP
for (NAME in names_df$NAME){

  # denovo list
  main_df <- read.table(paste("data/",NAME,"_main_dataframe.tsv", sep=""), header = TRUE)
  denovos_NCOS <- as.vector(main_df[main_df$NCOS == "TRUE" & ! is.na(main_df$NCOS), "seq_name"])
  denovos_NCO <- as.vector(main_df[main_df$NCO == "TRUE" & ! is.na(main_df$NCO), "seq_name"])

  # Match matrix
  match_matrix <- read.table(paste("data/TRG_match_matrix/",NAME,"_TRG_match_matrix.tsv", sep=""), header = TRUE)
  # Remove "_elongated" from the end of match_matrix$CDS
  match_matrix$CDS <- gsub("_elongated", "", match_matrix$CDS)
  # Only keep the denovo genes CDS
  match_matrix_NCOS <- match_matrix[match_matrix$CDS %in% denovos_NCOS, ]
  match_matrix_NCO <- match_matrix[match_matrix$CDS %in% denovos_NCO, ]

  # Replace genome names by short names in the colnames of match matrix
  names_mapping_df <- read.table(paste("data/names/",NAME,"_names.tsv", sep=""), sep = "\t")
  colnames(names_mapping_df) <- c("genome","short")
  # Find the indices of the long names in names_mapping_df$genome
  indices <- match(colnames(match_matrix_NCOS)[-1], names_mapping_df$genome)

  # Replace the long names with the corresponding short names
  colnames(match_matrix_NCOS)[-1] <- names_mapping_df$short[indices]
  colnames(match_matrix_NCO)[-1]  <- names_mapping_df$short[indices]
  # Remove the "CDS" column
  match_matrix_NCOS <- match_matrix_NCOS[, -1]
  match_matrix_NCO <- match_matrix_NCO[, -1]
  
  # Conservation
  # Calculate the percentage of "CDS" for each genome
  NCOS_conservation <- colSums(match_matrix_NCOS == "CDS") / nrow(match_matrix_NCOS) * 100
  NCO_conservation <-  colSums(match_matrix_NCO == "CDS") / nrow(match_matrix_NCO) * 100

  # Conservation dataframe
  conservation_df <- data.frame( genome = colnames(match_matrix_NCOS),
                                 NCOS = as.numeric(NCOS_conservation),
                                 NCO = as.numeric(NCO_conservation)
                                 )

  # Add the iORF column
  iORF_conservation_df <- read.csv(paste("data/iORF_conservation/",NAME,"_iORF_conservation_table.csv", sep=""))
  indices <- match(iORF_conservation_df$genome, names_mapping_df$genome)
  iORF_conservation_df$genome <- names_mapping_df$short[indices]
  conservation_df <- merge(conservation_df, iORF_conservation_df, by = "genome")
  print(conservation_df)

  # Add the IGRs column
  IGR_conservation_df <- read.csv(paste("data/IGR_conservation/",NAME,"_IGR_conservation_table.csv", sep=""))
  indices <- match(IGR_conservation_df$genome, names_mapping_df$genome)
  IGR_conservation_df$genome <- names_mapping_df$short[indices]
  conservation_df <- merge(conservation_df, IGR_conservation_df, by = "genome")
  print(conservation_df)


  # Tree
  tree_path <- paste("data/trees/",NAME,".nwk", sep="") # OF trees with focal at the top.
  tree <- read.tree(file = tree_path)
  tree$edge.length <- NULL # Remove branch length
  # Use the short names
  tree$tip.label <- names_mapping_df$short[match(tree$tip.label, names_mapping_df$genome)]
  leaf_order <- tree$tip.label

  # Manage labels=genomes=leaves
  for (i in 1:length(leaf_order)){
    if (!(leaf_order[i] %in% conservation_df$genome)){
      conservation_df <- rbind(conservation_df, c(leaf_order[i], 0, 0))
    }
  }
  conservation_df$genome <- factor(conservation_df$genome, levels = leaf_order, ordered = TRUE)
  conservation_df <- conservation_df[match(leaf_order, conservation_df$genome), ]
  
  plot1 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = NCOS))+
    geom_tile()+
    scale_fill_gradient2(low="#FAFAFA", high="#202020", limits = c(0, 100)) +
    labs(fill = "A: conservation of de novo strat1 + synteny") +
    guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
    theme_minimal()+
    theme(legend.position = "bottom",
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 6),
          legend.key.width = unit(1, "cm"),  # Adjust the width
          legend.key.height = unit(0.5, "cm"),  # Adjust the height
          legend.box = "vertical", legend.margin = margin(),
          axis.title = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank(),
          panel.grid = element_blank())+theme(plot.margin = margin(r=0.5))

  legend1 <- cowplot::get_legend(plot1)

  plot1 <- plot1 + theme(legend.position = "none")



  plot2 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = NCO))+
    geom_tile()+
    scale_fill_gradient2(low="#FAFAFA", high="#202020", limits = c(0, 100)) +
    labs(fill = "B: conservation of de novo strat1 - synteny") +
    guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
    theme_minimal()+
    theme(legend.position = "bottom",
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 6),
          legend.key.width = unit(1, "cm"),  # Adjust the width
          legend.key.height = unit(0.5, "cm"),  # Adjust the height
          legend.box = "vertical", legend.margin = margin(),
          axis.title = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank(),
          panel.grid = element_blank())+theme(plot.margin = margin(r=0.5))

  legend2 <- cowplot::get_legend(plot2)

  plot2 <- plot2 + theme(legend.position = "none")


  # plot3 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = iORF_rel))+
  #   geom_tile()+
  #   scale_fill_gradient2(low="#FAFAFA", high="#54067a", limits = c(0, 100)) +
  #   labs(fill = "C : conservation of noncoding ORFs           ") +
  #   guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
  #   theme_minimal()+
  #   theme(legend.position = "bottom",
  #         legend.title = element_text(size = 8),
  #         legend.text = element_text(size = 6),
  #         legend.key.width = unit(1, "cm"),  # Adjust the width
  #         legend.key.height = unit(0.5, "cm"),  # Adjust the height
  #         legend.box = "vertical", legend.margin = margin(),
  #         axis.title = element_blank(),
  #         axis.text = element_blank(),
  #         axis.ticks = element_blank(),
  #         panel.grid = element_blank())+theme(plot.margin = margin(l=0.5))
  
  # legend3 <- cowplot::get_legend(plot3)

  # plot3 <- plot3 + theme(legend.position = "none")

  plot3 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = IGR_rel))+
    geom_tile()+
    scale_fill_gradient2(low="#FAFAFA", high="#54067a", limits = c(0, 100)) +
    labs(fill = "C: conservation of noncoding segments     ") +
    guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
    theme_minimal()+
    theme(legend.position = "bottom",
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 6),
          legend.key.width = unit(1, "cm"),  # Adjust the width
          legend.key.height = unit(0.5, "cm"),  # Adjust the height
          legend.box = "vertical", legend.margin = margin(),
          axis.title = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank(),
          panel.grid = element_blank())+theme(plot.margin = margin(l=0.5, r=0.5))
  
  legend3 <- cowplot::get_legend(plot3)

  plot3 <- plot3 + theme(legend.position = "none")
  
  # Convert the tree to a data frame
  tree_df <- as.data.frame(tree$edge)
  tree_df$Label <- tree$tip.label[tree_df$V2]
  
  # Create a base R plot
  base_plot <- function() {
    plot(tree, no.margin = TRUE, cex = 0.95)
  }
  # Convert the base R plot to a ggplot object
  plot_tree <- as.ggplot(base_plot)+theme(plot.margin = margin(t=0, r=-1, b=0, l=0, "cm"))
  
  header_plot <- plot_grid(NULL, NULL, ggdraw()+draw_label("A", size = 10, x = 0.5, y = 0.5), ggdraw()+draw_label("B", size = 10, x = 0.5, y = 0.5), ggdraw()+draw_label("C", size = 10, x = 0.5, y = 0.5),
                          ncol = 5, scale = c(1,1,1,1,1),
                          rel_widths = c(rep(c(7, 0.5, 2/3,2/3, 2/3),1))
                          )
  footer_plot <- plot_grid(plot_tree, NULL, plot1,plot2,plot3,
                    ncol = 5, scale = c(1,1,1,1,1),
                    rel_widths = c(rep(c(7, 0.5, 2/3,2/3, 2/3),1))
                    )
  plot <- plot_grid(header_plot, footer_plot,
                    ncol = 1, scale = c(1,1),
                    rel_heights = c(header_height[[NAME]],30),
                    rel_widths = c(1)
                    )
  plot
  plots[[NAME]] <- plot

}

colnames(correlation_df) <- c("NAME","rho","p_value")

HM_plots <- plots
HM_plots[["LEGEND"]] <- plot_grid(legend1, legend2, legend3, nrow = 3)

# PDF
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "FigS1.pdf"), height = 18, width = 10)
HM_grid <- plot_grid(
                    NULL,NULL,
                    HM_plots[["SCER"]],HM_plots[["HSAP"]],
                    NULL,NULL,
                    HM_plots[["OSAT"]],HM_plots[["MMUS"]],
                    NULL,NULL,
                    HM_plots[["DMEL"]],HM_plots[["CELE"]],
                    NULL,NULL,
                    HM_plots[["ATHA"]],HM_plots[["LEGEND"]],
                    NULL,NULL,
                    rel_heights = c(
                    0.1,
                    0.66,
                    0.1,
                    1.25,
                    0.1,
                    2.5,
                    0.1,
                    0.5,
                    0.1),
                    ncol = 2)
HM_grid
dev.off()


################
#### Fig S2 ####
################


# This section generates the heatmap of the conservation of denovo CDS and iORF in the different species (Fig3).

plots <- list()

for (name in c("Oryza_sativa","Oryza_indica","Oryza_nivara","Oryza_rufipogon")){
  # denovo list
  dn_df <- read.table(paste("data/FigS2/",name,"_denovogenes.tsv", sep=""), header = TRUE)
  denovos <- as.vector(dn_df$CDS)

  # Match matrix
  match_matrix <- read.table(paste("data/FigS2/",name,"_TRG_match_matrix.tsv", sep=""), header = TRUE)
  # Remove "_elongated" from the end of match_matrix$CDS
  match_matrix$CDS <- gsub("_elongated", "", match_matrix$CDS)
  # Only keep the denovo genes CDS
  match_matrix <- match_matrix[match_matrix$CDS %in% denovos, ]

  # Replace genome names by short names in the colnames of match matrix
  names_mapping_df <- read.table(paste("data/names/OSAT_names.tsv", sep=""), sep = "\t")
  colnames(names_mapping_df) <- c("genome","short")
  # Find the indices of the long names in names_mapping_df$genome
  indices <- match(colnames(match_matrix)[-1], names_mapping_df$genome)
  # Replace the long names with the corresponding short names
  colnames(match_matrix)[-1] <- names_mapping_df$short[indices]
  # Remove the "CDS" column
  match_matrix <- match_matrix[, -1]

  # Conservation
  # Calculate the percentage of "CDS" for each genome
  CDS_conservation <- colSums(match_matrix == "CDS") / nrow(match_matrix) * 100

  # Conservation dataframe
  conservation_df <- data.frame( genome = colnames(match_matrix),
                                  CDS = as.numeric(CDS_conservation) )


  # Tree
  tree_path <- paste("data/trees/OSAT.nwk", sep="") # OF trees with focal at the top.
  tree <- read.tree(file = tree_path)
  tree$edge.length <- NULL # Remove branch length
  # Use the short names
  tree$tip.label <- names_mapping_df$short[match(tree$tip.label, names_mapping_df$genome)]
  leaf_order <- tree$tip.label

  # Manage labels=genomes=leaves
  for (i in 1:length(leaf_order)){
    if (!(leaf_order[i] %in% conservation_df$genome)){
      conservation_df <- rbind(conservation_df, c(leaf_order[i], 0, 0))
    }
  }
  conservation_df$genome <- factor(conservation_df$genome, levels = leaf_order, ordered = TRUE)
  conservation_df <- conservation_df[match(leaf_order, conservation_df$genome), ]
  write.table(conservation_df, paste("data/FigS2/",name,"_conservation.tsv", sep=""), sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE)

  plot1 <- ggplot(data = conservation_df, aes(x = 1, y = genome, fill = CDS))+
    geom_tile()+
    scale_fill_gradient2(low="#FAFAFA", high="#202020", limits = c(0, 100)) +
    labs(fill = "conservation of de novo emerged genes") +
    guides(fill = guide_colorbar(title.position = "bottom", title.hjust = 0.5))+
    theme_minimal()+
    theme(legend.position = "bottom",
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 6),
          legend.key.width = unit(1, "cm"),  # Adjust the width
          legend.key.height = unit(0.5, "cm"),  # Adjust the height
          legend.box = "vertical", legend.margin = margin(),
          axis.title = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank(),
          panel.grid = element_blank())+theme(plot.margin = margin(r=0.5))

  legend1 <- cowplot::get_legend(plot1)

  plot1 <- plot1 + theme(legend.position = "none")

  # Convert the tree to a data frame
  tree_df <- as.data.frame(tree$edge)
  tree_df$Label <- tree$tip.label[tree_df$V2]

  # Create a base R plot
  base_plot <- function() {
    plot(tree, no.margin = TRUE, cex = 0.95)
  }
  # Convert the base R plot to a ggplot object
  plot_tree <- as.ggplot(base_plot)#+theme(plot.margin = margin(t=1, r=1, b=1, l=1, "cm"))

  plot <- plot_grid(plot_tree,plot1, nrow = 1, scale = c(1,1), rel_widths = c(7, 1))
  plots[[name]] <- plot1
}

HM_plots <- plots

header_grid <- plot_grid(
  NULL,
  ggdraw()+draw_label("Osat", size = 10, x = 0.5, y = 0.5),
  ggdraw()+draw_label("Oind", size = 10, x = 0.5, y = 0.5),
  ggdraw()+draw_label("Oruf", size = 10, x = 0.5, y = 0.5),
  ggdraw()+draw_label("Oniv", size = 10, x = 0.5, y = 0.5),
  ncol = 5,
  rel_widths = c(10,1,1,1,1))
HM_grid <- plot_grid(
  plot_tree,
  HM_plots[["Oryza_sativa"]],
  HM_plots[["Oryza_indica"]],
  HM_plots[["Oryza_rufipogon"]],
  HM_plots[["Oryza_nivara"]],
  ncol = 5,
  rel_widths = c(10,1,1,1,1))
HM_grid <- plot_grid(
  header_grid,
  HM_grid,
  legend1,
  ncol = 1,
  rel_heights = c(0.1,2,0.2),
  rel_widths = c(1))

# PDF
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "FigS2.pdf"), width = 7)

HM_grid
dev.off()


##############
#### FigS3 ###
##############
# This script generates the features boxplots of the different species (Fig3).

# Function that remove the legend and the axis.
remove_legend_and_axis <- function(plot){
  plot <- plot + theme(legend.position = "none",
                        axis.title.x = element_blank(),
                        axis.text.x = element_blank(),
                        axis.ticks.x = element_blank(),
                        axis.title.y = element_blank(),
                        axis.text.y = element_blank(),
                        axis.ticks.y = element_blank())+
                  ggtitle(NULL)
  return(plot)
}
plots_to_one_line <- function(plot_list, y_title_text = "y_title"){
  
  # Get the y axis of the first plot.
  y_title_text <- plot_list[[1]]$labels$y
  y_title <- ggdraw()+draw_label(y_title_text, x = 0.2, y = 0.5, size = 6, angle = 90)
  y_axis <- get_y_axis(plot_list[[1]]+ theme(axis.text.y = element_text(size = 6)))

  # For each plot in plot_list, remove the legend and the axis.
  plot_list_formatted <- lapply(plot_list, remove_legend_and_axis)

  # Get the same order as names_df$NAME
  plot_list_formatted <- plot_list_formatted[names_df$NAME]

  # Unique list with the y title, the y axis and the plots.
  plotlist <- c(list(y_title, y_axis), plot_list_formatted)

  # All genomes on one line :
  oneline_plot <- plot_grid(plotlist = plotlist, nrow = 1, rel_widths = c(2,1,rep(5,length(plot_list_formatted))))

  return(oneline_plot)
}

comparisons <- list(c("NCO3S","NCO2S"),
                    c("NCO3S","NCOS"),
                    c("NCO3S","NCO"),
                    c("NCO3S","NC"),
                    c("NCO3S","TRG"),
                    c("NCO3S","nonTRG"))

#### SIZES ####
size_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"size plot..."))

  size_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                          feature = "size",
                                          groups = c("NCO3S", "NCO2S","NCOS","NCO","NC","TRG","nonTRG"),
                                          comps = comparisons,
                                          yp=seq(200, 500, length.out = length(comparisons)),
                                          signif_size = 2,
                                          num_outgroups = TRUE
                                          )+
                                          ylab("Length")
}

#### GAN ####
GAN_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"GAN plot..."))
  GAN_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                          feature = "GAN",
                                          groups = c("NCO3S", "NCO2S","NCOS","NCO","NC","TRG","nonTRG"),
                                          comps = comparisons,
                                          yp=seq(0.125, 0.3, length.out = length(comparisons)),
                                          signif_size = 2,
                                          num_outgroups = TRUE
                                          )+
                                          ylab("Fraction of negatively\ncharged residues")
}

#### gene_dist ####
gene_dist_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"gene_dist plot..."))
  gene_dist_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                                feature = "gene_dist",
                                                groups = c("NCO3S", "NCO2S","NCOS","NCO","NC","TRG","nonTRG"),
                                                comps = comparisons,
                                                yp=seq(350, 1000, length.out = length(comparisons)),
                                          signif_size = 2,
                                          num_outgroups = TRUE
                                          )+
                                                ylab("Distance to closest\ngene (bp)")
}

### dNdS_p ####
dNdS_p_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"dNdS_p plot..."))
  dNdS_p_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                            feature = "p_neg",
                                            groups = c("NCO3S", "NCO2S","NCOS","NCO","NC","TRG","nonTRG"),
                                            comps = comparisons,
                                            yp=seq(0.05, 0.5, length.out = length(comparisons)),
                                          signif_size = 2,
                                          num_outgroups = TRUE
                                          )+
                                            ylab("Fraction of residues\nunder neg. selection")
}

#### age ####
age_plots <- list()
for (NAME in names_df$NAME) {
# for (NAME in c("SCER","OSAT","HSAP","DMEL")) {
  print(paste("Generating",NAME,"age plot..."))
  age_comparisons <- list(c("NCO3S","NCO2S"),
                      c("NCO3S","NCOS"),
                      c("NCO3S","NCO"),
                      c("NCO3S","NC"),
                      c("NCO3S","TRG"))
  age_plots[[NAME]] <- get_exclusive_plot(NAME = NAME, 
                                          feature = "age",
                                          groups = c("NCO3S", "NCO2S","NCOS","NCO","NC","TRG"),
                                          comps = age_comparisons,
                                          yp=seq(0.15, 0.75, length.out = length(age_comparisons)),
                                          signif_size = 2,
                                          num_outgroups = TRUE
                                          )+
                                          ylab("Phylogenetic distance")
}

###############


sizes <- plots_to_one_line(size_plots)
GANs <- plots_to_one_line(GAN_plots)
gene_dists <- plots_to_one_line(gene_dist_plots)
dNdS_ps <- plots_to_one_line(dNdS_p_plots)
ages <- plots_to_one_line(age_plots)
names_plots <- plot_grid(plotlist = c( list(NULL,NULL), lapply(names_df$NAME, function(x) ggdraw()+draw_label(x, x = 0.5, y = 0.5, size = 10))),
                         nrow = 1,
                         rel_widths = c(2,1,rep(5,length(names_df$NAME))))
# names_plots <- plot_grid(plotlist = c( list(NULL,NULL), lapply(c("SCER","OSAT","HSAP","DMEL"), function(x) ggdraw()+draw_label(x, x = 0.5, y = 0.5, size = 10))),
#                          nrow = 1,
#                          rel_widths = c(2,1,rep(5,length(c("SCER","OSAT","HSAP","DMEL")))))
horizontal_spacer_size <- 1
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "FigS3.pdf"))

plot_grid(names_plots,
          NULL,
          sizes,
          NULL,
          GANs,
          NULL,
          gene_dists,
          NULL,
          dNdS_ps,
          NULL,
          ages,
          NULL,
          nrow = 12,
          rel_heights = c(1,horizontal_spacer_size,rep(c(5,horizontal_spacer_size),5)))

dev.off()


##############
#### FigS4 ###
##############
df <- read.table("data/FigS4/Osat_CDS_found_in_Zhang_2019.tsv", header = TRUE, sep="\t")
table(df$rank,df[,"found.in.Zhang_2019"])

df$found.in.Zhang_2019 <- as.factor(df$found.in.Zhang_2019)

# Calculate counts
df_summary_counts <- df %>%
  group_by(rank, found.in.Zhang_2019) %>%
  summarise(count = n())

# Plot
p_counts <- ggplot(df_summary_counts, aes(x = rank, y = count, fill = found.in.Zhang_2019)) +
  geom_bar(stat = "identity") +
  scale_fill_manual(values = c("TRUE" = "grey", "FALSE" = "#d85353")) +  # adjust colors as needed
  labs(x = "Phylostratum rank", y = "Number of CDSs", fill = "Shared with Stein 2018") +
  theme_minimal()+
  theme(axis.text.x = element_text(size = 16),
        axis.text.y = element_text(size = 16),
        axis.title.x = element_text(size = 16),
        axis.title.y = element_text(size = 16),
        legend.title = element_text(size = 16),
        legend.text = element_text(size = 12))

# Save the plot
pdf(file=paste0(format(Sys.time(), "%Y%m%d_%H%M%S_"), "FigS4.pdf"), width = 15)
print(p_counts)
dev.off()